@@ -24,9 +24,10 @@ from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSe
                                   CommentedKeyMap)
 from ruamel.yaml.scalarstring import (SingleQuotedScalarString, DoubleQuotedScalarString,
                                       LiteralScalarString, FoldedScalarString,
-                                      ScalarString,)
+                                      PlainScalarString, ScalarString,)
 from ruamel.yaml.scalarint import ScalarInt, BinaryInt, OctalInt, HexInt, HexCapsInt
 from ruamel.yaml.scalarfloat import ScalarFloat
+from ruamel.yaml.scalarbool import ScalarBoolean
 from ruamel.yaml.timestamp import TimeStamp
 from ruamel.yaml.util import RegExp
 
@@ -155,11 +156,11 @@ class BaseConstructor(object):
                 elif None in self.yaml_constructors:
                     constructor = self.yaml_constructors[None]
                 elif isinstance(node, ScalarNode):
-                    constructor = self.__class__.construct_scalar  # type: ignore
+                    constructor = self.__class__.construct_scalar
                 elif isinstance(node, SequenceNode):
-                    constructor = self.__class__.construct_sequence  # type: ignore
+                    constructor = self.__class__.construct_sequence
                 elif isinstance(node, MappingNode):
-                    constructor = self.__class__.construct_mapping  # type: ignore
+                    constructor = self.__class__.construct_mapping
         if tag_suffix is None:
             data = constructor(self, node)
         else:
@@ -241,13 +242,16 @@ class BaseConstructor(object):
 
                 value = self.construct_object(value_node, deep=deep)
                 if check:
-                    self.check_mapping_key(node, key_node, mapping, key, value)
-                mapping[key] = value
+                    if self.check_mapping_key(node, key_node, mapping, key, value):
+                        mapping[key] = value
+                else:
+                    mapping[key] = value
             total_mapping.update(mapping)
         return total_mapping
 
     def check_mapping_key(self, node, key_node, mapping, key, value):
-        # type: (Any, Any, Any, Any, Any) -> None
+        # type: (Any, Any, Any, Any, Any) -> bool
+        """return True if key is unique"""
         if key in mapping:
             if not self.allow_duplicate_keys:
                 mk = mapping.get(key)
@@ -277,6 +281,8 @@ class BaseConstructor(object):
                     warnings.warn(DuplicateKeyFutureWarning(*args))
                 else:
                     raise DuplicateKeyError(*args)
+            return False
+        return True
 
     def check_set_key(self, node, key_node, setting, key):
         # type: (Any, Any, Any, Any, Any) -> None
@@ -1052,7 +1058,7 @@ class RoundTripConstructor(SafeConstructor):
             )
 
         if node.style == '|' and isinstance(node.value, text_type):
-            lss = LiteralScalarString(node.value)
+            lss = LiteralScalarString(node.value, anchor=node.anchor)
             if node.comment and node.comment[1]:
                 lss.comment = node.comment[1][0]  # type: ignore
             return lss
@@ -1064,7 +1070,7 @@ class RoundTripConstructor(SafeConstructor):
                 if idx < 0:
                     break
                 fold_positions.append(idx - len(fold_positions))
-            fss = FoldedScalarString(node.value.replace('\a', ''))
+            fss = FoldedScalarString(node.value.replace('\a', ''), anchor=node.anchor)
             if node.comment and node.comment[1]:
                 fss.comment = node.comment[1][0]  # type: ignore
             if fold_positions:
@@ -1072,9 +1078,11 @@ class RoundTripConstructor(SafeConstructor):
             return fss
         elif bool(self._preserve_quotes) and isinstance(node.value, text_type):
             if node.style == "'":
-                return SingleQuotedScalarString(node.value)
+                return SingleQuotedScalarString(node.value, anchor=node.anchor)
             if node.style == '"':
-                return DoubleQuotedScalarString(node.value)
+                return DoubleQuotedScalarString(node.value, anchor=node.anchor)
+        if node.anchor:
+            return PlainScalarString(node.value, anchor=node.anchor)
         return node.value
 
     def construct_yaml_int(self, node):
@@ -1102,8 +1110,11 @@ class RoundTripConstructor(SafeConstructor):
             if underscore is not None:
                 underscore[1] = value_su[2] == '_'
                 underscore[2] = len(value_su[2:]) > 1 and value_su[-1] == '_'
-            return BinaryInt(  # type: ignore
-                sign * int(value_s[2:], 2), width=width, underscore=underscore
+            return BinaryInt(
+                sign * int(value_s[2:], 2),
+                width=width,
+                underscore=underscore,
+                anchor=node.anchor,
             )
         elif value_s.startswith('0x'):
             # default to lower-case if no a-fA-F in string
@@ -1119,15 +1130,23 @@ class RoundTripConstructor(SafeConstructor):
             if underscore is not None:
                 underscore[1] = value_su[2] == '_'
                 underscore[2] = len(value_su[2:]) > 1 and value_su[-1] == '_'
-            return hex_fun(sign * int(value_s[2:], 16), width=width, underscore=underscore)
+            return hex_fun(
+                sign * int(value_s[2:], 16),
+                width=width,
+                underscore=underscore,
+                anchor=node.anchor,
+            )
         elif value_s.startswith('0o'):
             if self.resolver.processing_version > (1, 1) and value_s[2] == '0':
                 width = len(value_s[2:])
             if underscore is not None:
                 underscore[1] = value_su[2] == '_'
                 underscore[2] = len(value_su[2:]) > 1 and value_su[-1] == '_'
-            return OctalInt(  # type: ignore
-                sign * int(value_s[2:], 8), width=width, underscore=underscore
+            return OctalInt(
+                sign * int(value_s[2:], 8),
+                width=width,
+                underscore=underscore,
+                anchor=node.anchor,
             )
         elif self.resolver.processing_version != (1, 2) and value_s[0] == '0':
             return sign * int(value_s, 8)
@@ -1145,15 +1164,15 @@ class RoundTripConstructor(SafeConstructor):
             if underscore is not None:
                 # cannot have a leading underscore
                 underscore[2] = len(value_su) > 1 and value_su[-1] == '_'
-            return ScalarInt(  # type: ignore
-                sign * int(value_s), width=len(value_s), underscore=underscore
-            )
+            return ScalarInt(sign * int(value_s), width=len(value_s), underscore=underscore)
         elif underscore:
             # cannot have a leading underscore
             underscore[2] = len(value_su) > 1 and value_su[-1] == '_'
-            return ScalarInt(  # type: ignore
-                sign * int(value_s), width=None, underscore=underscore
+            return ScalarInt(
+                sign * int(value_s), width=None, underscore=underscore, anchor=node.anchor
             )
+        elif node.anchor:
+            return ScalarInt(sign * int(value_s), width=None, anchor=node.anchor)
         else:
             return sign * int(value_s)
 
@@ -1211,7 +1230,7 @@ class RoundTripConstructor(SafeConstructor):
             e_width = len(exponent)
             e_sign = exponent[0] in '+-'
             # nprint('sf', width, prec, m_sign, exp, e_width, e_sign)
-            return ScalarFloat(  # type: ignore
+            return ScalarFloat(
                 sign * float(value_s),
                 width=width,
                 prec=prec,
@@ -1220,12 +1239,18 @@ class RoundTripConstructor(SafeConstructor):
                 exp=exp,
                 e_width=e_width,
                 e_sign=e_sign,
+                anchor=node.anchor,
             )
         width = len(value_so)
         prec = value_so.index('.')  # you can use index, this would not be float without dot
         lead0 = leading_zeros(value_so)
-        return ScalarFloat(  # type: ignore
-            sign * float(value_s), width=width, prec=prec, m_sign=m_sign, m_lead0=lead0
+        return ScalarFloat(
+            sign * float(value_s),
+            width=width,
+            prec=prec,
+            m_sign=m_sign,
+            m_lead0=lead0,
+            anchor=node.anchor,
         )
 
     def construct_yaml_str(self, node):
@@ -1411,31 +1436,31 @@ class RoundTripConstructor(SafeConstructor):
                         key_node.start_mark,
                     )
             value = self.construct_object(value_node, deep=deep)
-            self.check_mapping_key(node, key_node, maptyp, key, value)
+            if self.check_mapping_key(node, key_node, maptyp, key, value):
 
-            if key_node.comment and len(key_node.comment) > 4 and key_node.comment[4]:
-                if last_value is None:
-                    key_node.comment[0] = key_node.comment.pop(4)
-                    maptyp._yaml_add_comment(key_node.comment, value=last_key)
-                else:
-                    key_node.comment[2] = key_node.comment.pop(4)
+                if key_node.comment and len(key_node.comment) > 4 and key_node.comment[4]:
+                    if last_value is None:
+                        key_node.comment[0] = key_node.comment.pop(4)
+                        maptyp._yaml_add_comment(key_node.comment, value=last_key)
+                    else:
+                        key_node.comment[2] = key_node.comment.pop(4)
+                        maptyp._yaml_add_comment(key_node.comment, key=key)
+                    key_node.comment = None
+                if key_node.comment:
                     maptyp._yaml_add_comment(key_node.comment, key=key)
-                key_node.comment = None
-            if key_node.comment:
-                maptyp._yaml_add_comment(key_node.comment, key=key)
-            if value_node.comment:
-                maptyp._yaml_add_comment(value_node.comment, value=key)
-            maptyp._yaml_set_kv_line_col(
-                key,
-                [
-                    key_node.start_mark.line,
-                    key_node.start_mark.column,
-                    value_node.start_mark.line,
-                    value_node.start_mark.column,
-                ],
-            )
-            maptyp[key] = value
-            last_key, last_value = key, value  # could use indexing
+                if value_node.comment:
+                    maptyp._yaml_add_comment(value_node.comment, value=key)
+                maptyp._yaml_set_kv_line_col(
+                    key,
+                    [
+                        key_node.start_mark.line,
+                        key_node.start_mark.column,
+                        value_node.start_mark.line,
+                        value_node.start_mark.column,
+                    ],
+                )
+                maptyp[key] = value
+                last_key, last_value = key, value  # could use indexing
         # do this last, or <<: before a key will prevent insertion in instances
         # of collections.OrderedDict (as they have no __contains__
         if merge_map:
@@ -1505,7 +1530,7 @@ class RoundTripConstructor(SafeConstructor):
         data = CommentedMap()
         data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
         yield data
-        self.construct_mapping(node, data)
+        self.construct_mapping(node, data, deep=True)
         self.set_collection_style(data, node)
 
     def set_collection_style(self, data, node):
@@ -1595,6 +1620,8 @@ class RoundTripConstructor(SafeConstructor):
                     data.fa.set_block_style()
                 data.yaml_set_tag(node.tag)
                 yield data
+                if node.anchor:
+                    data.yaml_set_anchor(node.anchor)
                 self.construct_mapping(node, data)
                 return
             elif isinstance(node, ScalarNode):
@@ -1603,6 +1630,8 @@ class RoundTripConstructor(SafeConstructor):
                 data2.style = node.style
                 data2.yaml_set_tag(node.tag)
                 yield data2
+                if node.anchor:
+                    data2.yaml_set_anchor(node.anchor, always_dump=True)
                 return
             elif isinstance(node, SequenceNode):
                 data3 = CommentedSeq()
@@ -1613,6 +1642,8 @@ class RoundTripConstructor(SafeConstructor):
                     data3.fa.set_block_style()
                 data3.yaml_set_tag(node.tag)
                 yield data3
+                if node.anchor:
+                    data3.yaml_set_anchor(node.anchor)
                 data3.extend(self.construct_sequence(node))
                 return
         except:  # NOQA
@@ -1685,6 +1716,13 @@ class RoundTripConstructor(SafeConstructor):
             data._yaml['t'] = True
         return data
 
+    def construct_yaml_bool(self, node):
+        # type: (Any) -> Any
+        b = SafeConstructor.construct_yaml_bool(self, node)
+        if node.anchor:
+            return ScalarBoolean(b, anchor=node.anchor)
+        return b
+
 
 RoundTripConstructor.add_constructor(
     u'tag:yaml.org,2002:null', RoundTripConstructor.construct_yaml_null
